package com.yizhi.message.application.controller.api;


import com.baomidou.mybatisplus.mapper.EntityWrapper;
import com.baomidou.mybatisplus.plugins.Page;
import com.yizhi.application.orm.id.IdGenerator;
import com.yizhi.core.application.context.ContextHolder;
import com.yizhi.core.application.context.RequestContext;
import com.yizhi.message.application.constans.Constans;
import com.yizhi.message.application.controller.util.MessageUtil;
import com.yizhi.message.application.domain.Message;
import com.yizhi.message.application.domain.MessageAccount;
import com.yizhi.message.application.domain.MessageReadRecord;
import com.yizhi.message.application.mapper.MessageAccountMapper;
import com.yizhi.message.application.service.IMessageReadRecordService;
import com.yizhi.message.application.service.MessageAccountService;
import com.yizhi.message.application.service.MessageRemindService;
import com.yizhi.message.application.service.MessageService;
import com.yizhi.util.application.domain.Response;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * <p>
 * 前端控制器 用户消息记录表
 * </p>
 *
 * @author hutao123
 * @since 2019-09-09
 */
@RestController
@Api(tags = "学员端消息接口")
@RequestMapping("/api/messageAccount")
public class MessageAccountComtroller {
    @Autowired
    private MessageAccountService messageAccountService;
    @Autowired
    private MessageRemindService messageRemindService;
    @Autowired
    private MessageService messageService;
    @Autowired
    private IMessageReadRecordService messageReadRecordService;
    @Autowired
    private MessageUtil messageUtil;
    @Autowired
    private IdGenerator idGenerator;
    @Autowired
    private MessageAccountMapper messageAccountMapper;

    /**
     * 获取用户未读消息的个数
     *
     * @return
     */
    @GetMapping("/unread/count")
    @ApiOperation(value = "获取用户未读消息的个数", response = Integer.class)
    public Response unread() {
        RequestContext context = ContextHolder.get();
        Integer unReadNum = 0;
        EntityWrapper wrapper = new EntityWrapper();
        wrapper.eq("deleted", Constans.NOT_DELETED)
                .eq("site_id", context.getSiteId())
                .eq("account_id", context.getAccountId())
                .eq("readed", Constans.NOT_READ);
        unReadNum = messageAccountService.selectCount(wrapper);
        if (unReadNum > 0) {
            return Response.ok(unReadNum);
        } else {
            EntityWrapper wrapper1 = new EntityWrapper();
            wrapper1.eq("deleted", Constans.NOT_DELETED)
                    .eq("site_id", context.getSiteId())
                    .eq("visible_range", Constans.PUBLIC_SITE);
            unReadNum = messageAccountService.selectCount(wrapper1);
        }
        Integer readNum = messageReadRecordService.getReadNum();
        unReadNum = unReadNum - readNum;
        return Response.ok(unReadNum > 0 ? unReadNum : 0);
    }

    /**
     * 获取用户消息列表   弃用
     *
     * @return
     */
    @GetMapping("/list")
    @ApiOperation(value = "获取用户消息列表", response = Integer.class)
    @Deprecated
    public Response list() {
        RequestContext context = ContextHolder.get();
        EntityWrapper wrapper = new EntityWrapper();
        wrapper.eq("deleted", Constans.NOT_DELETED)
                .eq("site_id", context.getSiteId())
                .eq("account_id", context.getAccountId())
                .orderBy("create_time", false);
        return Response.ok(messageAccountService.selectList(wrapper));
    }

    /**
     * 获取用户消息列表
     *
     * @return
     */
    @GetMapping("/page")
    @ApiOperation(value = "获取用户消息列表（分页）", response = Integer.class)
    public Response page(@ApiParam("当前页数") @RequestParam(value = "pageNo", defaultValue = "1", required = false) Integer pageNo,
                         @ApiParam("页内条数") @RequestParam(value = "pageSize", defaultValue = "10", required = false) Integer pageSize) {
        RequestContext context = ContextHolder.get();
        Page<MessageAccount> page = new Page(pageNo, pageSize);

        List<MessageAccount> messageAccounts = messageAccountMapper.getList(context.getSiteId(), context.getAccountId(), page);
        page.setRecords(messageAccounts);
        page.setTotal(messageAccountMapper.getListNum(context.getSiteId(),context.getAccountId()));


        if (CollectionUtils.isNotEmpty(messageAccounts)) {
            List<Long> messageAccountIds = page.getRecords().parallelStream().map(a -> a.getId()).collect(Collectors.toList());
            List<Long> messageIds = page.getRecords().parallelStream().map(a -> a.getMessageId()).collect(Collectors.toList());
            //方便判断该消息是否已读
            Map<Long, Long> readRecordsMap = messageReadRecordService.getMap(messageAccountIds);
            //获取消息title
            Map<Long, String> titleMap = messageService.getMessageTitleMap(messageIds);
            for (MessageAccount a : page.getRecords()) {
                //  用户名称给赋值,业务参数赋值已在业务保存待发消息时已处理
                String completeContext = messageUtil.getCompleteContext(a.getContext(), context);
                a.setContext(completeContext);
                if (titleMap != null) {
                    a.setTitle(titleMap.get(a.getMessageId()));
                }
                if (readRecordsMap != null) {
                    if (a.getVisibleRange().equals(Constans.PUBLIC_SITE)) {
                        a.setReaded(readRecordsMap.containsKey(a.getId()) == true ? 1 : 0);
                    }
                }
            }
        }
        return Response.ok(page);
    }


    /**
     * 点击，使之全部已读
     *
     * @return
     */
    @GetMapping("/read")
    @ApiOperation(value = "一键已读功能", response = Integer.class)
    public Response read() {
        RequestContext context = ContextHolder.get();
        MessageAccount messageAccount = new MessageAccount();
        messageAccount.setReaded(Constans.READED);
        messageAccount.setReadTime(new Date());
        EntityWrapper wrapper = new EntityWrapper();
        wrapper.setSqlSelect("id,visible_range");
        wrapper.eq("deleted", Constans.NOT_DELETED)
                .eq("site_id", context.getSiteId())
                .andNew()
                .eq("account_id", context.getAccountId())
                .eq("readed", Constans.NOT_READ)
                .or()
                .eq("visible_range", Constans.PUBLIC_SITE);
        List<MessageAccount> list = messageAccountService.selectList(wrapper);
        //先查询到未读记录的集合
        if (CollectionUtils.isNotEmpty(list)) {
            messageAccountService.update(messageAccount, wrapper);
            List<Long> messageAccountIds = list.parallelStream().map(a -> a.getId()).collect(Collectors.toList());
            //方便判断该消息是否已读
            Map<Long, Long> readRecordsMap = messageReadRecordService.getMap(messageAccountIds);
            List<MessageReadRecord> readRecords = new ArrayList<>(list.size());
            for (MessageAccount a : list) {
                if (readRecordsMap != null) {
                    if (!readRecordsMap.containsKey(a.getId())) {
                        buildReadRecords(readRecords, a, context);
                    }
                } else {
                    buildReadRecords(readRecords, a, context);
                }
            }
            //插入阅读记录
            messageReadRecordService.batchInsertReadRecord(readRecords);
        }
        return Response.ok();
    }

    /**
     * 构建阅读记录
     *
     * @param readRecords
     * @param messageAccount
     * @param context
     */
    public void buildReadRecords(List<MessageReadRecord> readRecords, MessageAccount messageAccount, RequestContext context) {
        MessageReadRecord readRecord = new MessageReadRecord();
        readRecord.setId(idGenerator.generate());
        readRecord.setMessageAccountId(messageAccount.getId());
        readRecord.setVisibleRange(messageAccount.getVisibleRange());
        readRecord.setAccountId(context.getAccountId());
        readRecord.setCreateByName(context.getAccountName());
        readRecord.setReaded(Constans.READED);
        readRecord.setReadTime(new Date());
        readRecords.add(readRecord);
    }

    @GetMapping("/getOne")
    @ApiOperation(value = "获取单条消息", response = MessageAccount.class)
    public Response<MessageAccount> getOne(@ApiParam("id必传") @RequestParam("id") Long id) {
        MessageAccount a = messageAccountService.selectById(id);
        if (a != null) {
            MessageAccount messageAccount = new MessageAccount();
            messageAccount.setId(id);
            messageAccount.setReaded(Constans.READED);
            messageAccount.setReadTime(new Date());
            messageAccountService.updateById(messageAccount);
            //增加阅读记录
            messageReadRecordService.insertReadRecord(messageAccount);

            Message message = messageService.selectById(a.getMessageId());
            if (message != null) {
                a.setTitle(message.getTitle());
                //  用户名称给赋值,业务参数赋值已在业务保存待发消息时已处理
                String completeContext = messageUtil.getCompleteContext(a.getContext(), ContextHolder.get());
                a.setContext(completeContext);
            }
        }
        return Response.ok(a);
    }

}

