package com.yizhi.message.application.controller.manage;


import com.baomidou.mybatisplus.plugins.Page;
import com.yizhi.message.application.domain.Message;
import com.yizhi.message.application.enums.MessageStatus;
import com.yizhi.message.application.service.MessageService;
import com.yizhi.message.application.vo.CancelSendMessageVo;
import com.yizhi.message.application.vo.MessageManageVo;
import com.yizhi.message.application.vo.MessageVo;
import com.yizhi.util.application.domain.Response;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * <p>
 * 消息 前端控制器 消息主体表
 * </p>
 *
 * @author hutao123
 * @since 2019-09-09
 */
@RestController
@Api(tags = "主体消息管理端接口")
@RequestMapping("/manage/message")
public class MessageComtroller {
    @Autowired
    private MessageService messageService;


    /**
     * 保存消息
     *
     * @param messageVo
     * @return
     */

    @PostMapping("/save")
    @ApiOperation(value = "新增消息", response = String.class)
    public Response save(@ApiParam("messageVo") @RequestBody MessageVo messageVo) {
        if (messageVo.getMessageType() == null || messageVo.getTitle() == null || messageVo.getContext() == null) {
            return Response.fail("参数不全！");
        } else if (messageVo.getTitle().trim() == "" || messageVo.getContext().trim() == "") {
            return Response.fail("消息标题或内容不可为空！");
        } else {
            switch (messageVo.getMessageType()) {
                case 1:
                    if (messageVo.getSendType() == null || messageVo.getSendTime() == null || messageVo.getVisibleRange() == null) {
                        return Response.fail("必填项必填！");
                    }
                    break;
                case 2:
                    if (messageVo.getTemplateName() == null || messageVo.getTemplateName().trim() == "") {
                        return Response.fail("消息模板名称不可为空!");
                    }
                    break;
                case 3:
                    if (messageVo.getEventName() == null || messageVo.getEventName().trim() == "") {
                        return Response.fail("触发事件名称不可为空！");
                    }
                    break;
            }
        }
        switch (messageService.save(messageVo)) {
            case 1:
                return Response.ok("消息保存成功");

            case -1:
                return Response.fail("消息保存失败");

            case -2:
                return Response.fail("保存主体消息异常");

            case -3:
                return Response.fail("保存消息内容异常");
        }

        return Response.fail("消息保存失败");
    }

    /**
     * 分页查询
     *
     * @param messageType
     * @param title
     * @param pageNo
     * @param pageSize
     * @return
     */
    @ApiOperation(value = "消息分页查询", response = MessageManageVo.class)
    @GetMapping("/page")
    public Response<Page<MessageManageVo>> getPage(@ApiParam("消息类型") @RequestParam(value = "messageType", required = false) Integer messageType,
                                                   @ApiParam("消息标题 ") @RequestParam(value = "title", required = false) String title,
                                                   @ApiParam("当前页码") @RequestParam(value = "pageNo", required = false, defaultValue = "1") Integer pageNo,
                                                   @ApiParam("页内条数") @RequestParam(value = "pageSize", required = false, defaultValue = "10") Integer pageSize) {
        Page<MessageManageVo> page = new Page<>(pageNo, pageSize);
        if (StringUtils.isBlank(title)) {
            if (messageType == null) {
                return Response.fail("分页查询，缺少messageType参数！");
            }
        }
        return Response.ok(messageService.getPage(messageType, title, page));
    }

    /**
     * 回显需要
     *
     * @param id
     * @return
     */
    @GetMapping("/get")
    @ApiOperation(value = "单条消息查询", response = MessageVo.class)
    public Response<MessageVo> get(@ApiParam("消息id 必传") @RequestParam("id") Long id) {

        return Response.ok(messageService.get(id));
    }

    /**
     * 修改消息
     *
     * @param messageVo
     * @return
     */
    @PostMapping("/update")
    @ApiOperation(value = "消息修改", response = String.class)
    public Response update(@ApiParam("messageVo") @RequestBody MessageVo messageVo) {
        if (messageVo.getId() == null) {
            return Response.fail("消息id必传！");
        }
        if (messageVo.getMessageType() == null) {
            return Response.fail("消息类型必传");
        }
        switch (messageService.update(messageVo)) {
            case 1:
                return Response.ok("消息修改成功");
            case -1:
                return Response.fail("消息修改失败");
            case -2:
                return Response.fail("修改消息内容异常");
            case -3:
                return Response.fail("数据库没有该消息");
            case -4:
                return Response.fail("非草稿状态的自定义消息则无法修改！");
        }
        return Response.fail("消息修改失败");
    }

    /**
     * 上下架
     *
     * @param vo
     * @return
     */
    @PostMapping("/upOrDown")
    @ApiOperation(value = "消息上下架", response = String.class)
    public Response upOrDown(@ApiParam("vo") @RequestBody CancelSendMessageVo vo) {
        if (vo.getId() == null || vo.getStatus() == null) {
            return Response.fail("缺少必要参数!");
        }

        if (MessageStatus.UP.getCode() != (vo.getStatus()) && MessageStatus.DOWN.getCode() != (vo.getStatus())) {
            return Response.fail("status值不合理！");
        }

        Integer num = messageService.upOrDown(vo);
        if (num == 1) {
            if (MessageStatus.DOWN.equals(vo.getStatus())) {
                return Response.ok("下架成功");
            }
            return Response.ok("上架成功");
        } else if (num == -2) {
            return Response.fail("该消息不支持上下架!");
        }
        return Response.fail("操作失败");
    }

    /**
     * 取消发送或者撤回
     * 取消发送需要删除定时任务
     *
     * @param vo
     * @return
     */
    @PostMapping("/cancel")
    @ApiOperation(value = "取消发送或者撤回", response = String.class)
    public Response cancel(@ApiParam("vo") @RequestBody CancelSendMessageVo vo) {
        if (vo.getId() == null || vo.getType() == null) {
            return Response.fail("缺少必要参数!");
        }
        Integer cancel = messageService.cancel(vo);
        if (cancel == 1) {
            return Response.ok("操作成功");
        } else if (cancel == -3) {
            return Response.fail("数据库没有该数据！");
        } else if (cancel == -2) {
            return Response.fail("该消息类型不支持撤回或取消发送");
        } else if (cancel == -5) {
            return Response.fail("没有查询到对应待发消息");
        }
        return Response.fail("操作失败");
    }

    /**
     * 主要用到vo内的id必传
     *
     * @param vo
     * @return
     */
    @PostMapping("/send")
    @ApiOperation(value = "发送消息", response = String.class)
    public Response send(@ApiParam("vo") @RequestBody CancelSendMessageVo vo) {
        if (vo.getId() == null) {
            return Response.fail("缺少必要参数!");
        }
        Integer send = messageService.send(vo);
        if (send == 1) {
            return Response.ok("成功添加定时发送消息任务");
        } else if (send == -2) {
            return Response.fail("该消息类型不支持发送");
        } else if (send == -3) {
            return Response.fail("请选择未来的时间！");
        } else if (send == -4) {
            return Response.fail("数据库没有该数据！");
        }
        return Response.fail("操作失败");
    }


    /**
     * 主要用到vo内的id必传
     * 删除自定义任务
     *
     * @param vo
     * @return
     */
    @PostMapping("/deleted")
    @ApiOperation(value = "删除消息", response = String.class)
    public Response deleted(@ApiParam("vo") @RequestBody CancelSendMessageVo vo) {
        if (vo.getId() == null) {
            return Response.fail("id不能为空！");
        }
        Message message = this.messageService.selectById(vo.getId());
        if (message == null) {
            return Response.fail("删除失败，数据库没有该数据！");
        }
        if (message.getMessageType() == 1 && message.getStatus() == MessageStatus.DRAFT.getCode()) {
            message.setDeleted(1);
            if (this.messageService.updateById(message)) {
                return Response.ok("操作成功");
            }
        } else {
            return Response.fail("非草稿状态的自定义消息则无法删除！");
        }

        return Response.fail("操作失败");
    }

    /**
     * 业务那边需要系统消息列表
     * (应用场景)1：学习计划、2：考试、3：调研、4、投票5：报名、6：作业、7：签到、8：项目、9：直播、10：积分
     *
     * @param relationType
     * @return
     */
    @GetMapping("/getList")
    @ApiOperation(value = "业务新增时，模板消息列表查询", response = MessageVo.class)
    public Response<List<MessageVo>> getListByRelationType(@ApiParam("relationType 必传")
                                                           @RequestParam("relationType") Integer relationType) {

        return Response.ok(messageService.getListByRelationType(relationType));
    }


}

